/**
 * products.js
 * Copyright (C) 2007-2011 Tommi Rautava
 * 
 * This file is part of Popomungo.
 *
 * Popomungo is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Popomungo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

org.mozdev.popomungo.products = {
	
	LOW_STOCK_ALERT_CLASS: "popomungo_lowStockAlert",
	LOW_STOCK_ALERT_ICON_CLASS: "popomungo_lowStockAlertIcon",
	
	LOW_STOCK_WARNING_CLASS: "popomungo_lowStockWarning",
	LOW_STOCK_WARNING_ICON_CLASS: "popomungo_lowStockWarningIcon",
	
	
	highlightEmptyStock:
	function highlightEmptyStock(aDocument, aXpath) {
		try {
			if (!org.mozdev.popomungo.prefs.isEnabled("extensions.popomungo.options.highlighting.highlightLowStock.enabled")) {
				return;
			}
	
			var alertLevel = org.mozdev.popomungo.prefs.getPref("extensions.popomungo.options.products.lowStock.alertLevel", 0);
			var warningLevel = org.mozdev.popomungo.prefs.getPref("extensions.popomungo.options.products.lowStock.warningLevel", 9);
	
			var elems = org.mozdev.popomungo.commonFunctions.XPathOrderedSnapshot(aXpath, aDocument);
			
			org.mozdev.popomungo.logger.debug('elements='+ elems.snapshotLength);
		
			var i;
			// Loop through all items, but skip header.
			for (i = elems.snapshotLength - 1; i > 0; i--) {
				var elem = elems.snapshotItem(i);
				var stock = parseInt(elem.textContent);
	
				org.mozdev.popomungo.logger.debug('stock='+ stock +' (textContent='+ elem.textContent +', text='+ elem.text +')');
				
				var iconUrl;
				
				// Alert
				if (stock <= alertLevel) {
					iconUrl = aDocument.createElement('img');
					iconUrl.src = org.mozdev.popomungo.commonConstants.ALARM_ICON_URL;
					iconUrl.className = this.LOW_STOCK_ALERT_ICON_CLASS;
					//TODO iconUrl.title = org.mozdev.popomungo.strings.getString(org.mozdev.popomungo.stringKeys.XYZ, null);
					elem.className = elem.className ? ' '+ this.LOW_STOCK_ALERT_CLASS : this.LOW_STOCK_ALERT_CLASS;
					elem.insertBefore(iconUrl, elem.firstChild);
				}
				// Warning
				else if (stock <= warningLevel) {
					iconUrl = aDocument.createElement('img');
					iconUrl.src = org.mozdev.popomungo.commonConstants.WARNING_ICON_URL;
					iconUrl.className = this.LOW_STOCK_WARNING_ICON_CLASS;
					//TODO iconUrl.title = org.mozdev.popomungo.strings.getString(org.mozdev.popomungo.stringKeys.XYZ, null);
					elem.className = elem.className ? ' '+ this.LOW_STOCK_WARNING_CLASS : this.LOW_STOCK_WARNING_CLASS;
					elem.insertBefore(iconUrl, elem.firstChild);
				}
			}
		}
		catch (err) {
			org.mozdev.popomungo.logger.logError(err);
		}
	},


	highlightEmptyStockOnRestock: 
	function highlightEmptyStockOnRestock(aDocument) {
		var xpath1 = "/html/body/table[3]/tbody/tr/td[1]/table[2]/tbody/tr/td/form/table/tbody/tr/td[2]";

		this.highlightEmptyStock(aDocument, xpath1);
	},
	
	
	highlightEmptyStockOnManageProducts: 
	function highlightEmptyStockOnManageProducts(aDocument) {
		var xpath1 = "/html/body/table[3]/tbody/tr/td[1]/table[2]/tbody/tr/td/form/table/tbody/tr/td[3]";

		this.highlightEmptyStock(aDocument, xpath1);
	},
	
	
	highlightEmptyStockIfFormPage:
	function highlightEmptyStockIfFormPage(aDocument) {
		try {
			var form3 = aDocument.getElementsByName('form3').item(0);
			
			if (form3) {
				var action = form3.elements.namedItem('action').value.toLowerCase();
				org.mozdev.popomungo.logger.debug('action='+ action);
				
				if (action == 'updatestock') {
					this.highlightEmptyStockOnRestock(aDocument);
				}
				else if (action == 'updateprice') {
					this.highlightEmptyStockOnManageProducts(aDocument);
				}
			}
		}
		catch (err) {
			org.mozdev.popomungo.logger.logError(err);
		}
	}
};
